import { createRouter, createWebHistory } from 'vue-router';
import store from '@/stores/counter';
import axios from 'axios';
// 公共
import common from './module/common';
// 管理员
import admin from './module/admin';
// 系统管理
import system from './module/system';
// 账号管理
import account from './module/account';

const router = createRouter({
  history: createWebHistory(import.meta.env.BASE_URL),
  routes: [
    // 公共
    ...common,
    {
      path: '/',
      redirect: '/homeIndex'
    },
    {
      path: '/homeIndex',
      meta: { title: '中科在线（长春）' },
      component: () => import('@common/src/components/admin-frame/home.vue'),
      children: [
        {
          path: '/homeIndex',
          meta: { title: '中科在线（长春）' },
          component: () => import('@/views/home/index.vue')
        },
        ...admin,
        // 系统管理
        ...system,
        // 账号管理
        ...account
      ]
    }
  ]
});
router.beforeEach(async (to, from, next) => {
  document.title = `${to.meta.title} `;
  const token = localStorage.getItem('token');
  if (token) {
    const res = await axios.request({
      method: 'get',
      url: '/zkzx/v2/api/token/tokenView',
      responseType: 'json',
      headers: {
        token: token
      }
    });
    if (res.data.errcode == '0') {
      store.commit('setUser', res.data.data, { root: true });
    }
    next();
  } else {
    window.alert('无登录信息，无法打开');
  }
});

export default router;
